package cloudduggu.com;

import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;

import java.sql.Date;
import java.text.SimpleDateFormat;


/**
 * AnalysisLogic program implements an JavaRDD and
 * process spark query.
 *
 * @author  Sarvesh Kumar (CloudDuggu.com)
 * @version 1.0
 * @since   2020-07-25
 */

public final class AnalysisLogic
{

	Dataset<Row> process(String filePath, String query) throws Exception
	{
		SparkSession spark = SparkSession
			.builder()
			.appName("CloudDuggu Spark SQL Project")
			.getOrCreate();

//		Create an RDD of Movie objects from a csv file
		JavaRDD<Movie> movieRDD = spark.read()
			.csv(filePath)
			.javaRDD()
			.map(line -> {
				Movie movie = new Movie();
				movie.setMovieId(Integer.parseInt(line.getString(0)));
				movie.setTitle(line.getString(1));
				movie.setBudget(Long.parseLong(line.getString(2)));
				movie.setGross(Long.parseLong(line.getString(3)));
				movie.setReleaseDate(new Date(new SimpleDateFormat("yyyy-MM-dd").parse(line.getString(4)).getTime()));
				movie.setGenre(line.getString(5));
				movie.setRating(Float.parseFloat(line.getString(6)));
				movie.setSummary(line.getString(7));
				return movie;
			});

//		Apply a schema to an RDD of JavaBeans to get a DataFrame
		Dataset<Row> peopleDF = spark.createDataFrame(movieRDD, Movie.class);

// 		Register the DataFrame as a temporary view
		peopleDF.createOrReplaceTempView("movie");

// 		SQL statements can be run by using the sql methods provided by spark
		Dataset<Row> teenagersDF = spark.sql(query);

		return teenagersDF;
	}

}