package cloudduggu.com;

import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

/**
 * MovieAnalysis program for accept request from client and send result to client.
 *
 * @author  Sarvesh Kumar (CloudDuggu.com)
 * @version 1.0
 * @since   2020-07-25
 */

public final class MovieAnalysis
{
	private static final Logger logger = LoggerFactory.getLogger(MovieAnalysis.class);

	public final static void main(final String[] args)
	{
		try {
			AnalysisLogic analysisLogic = new AnalysisLogic();

            StringBuffer requestQuery = new StringBuffer();

//          Download Request Query
			String queryUrl = "http://"+args[1]+":"+args[2]+"/getQuery";
			URL reqUrl = new URL(queryUrl);
			HttpURLConnection reqConn = (HttpURLConnection) reqUrl.openConnection();
            reqConn.setRequestMethod("GET");
			if (reqConn.getResponseCode() == HttpURLConnection.HTTP_OK) { // success
                BufferedReader in = new BufferedReader(new InputStreamReader(reqConn.getInputStream()));
                String inputLine;
                while ((inputLine = in.readLine()) != null) {
                    requestQuery.append(inputLine);
                }
                in.close();
            } else {
				logger.info("==============================================================");
				logger.info("GET request not worked");
				logger.info("==============================================================");
            }


//          Spark Process Request Query
			logger.info("==============================================================");
			logger.info("DataFile: "+args[0]+ " | Query: "+requestQuery.toString());
			logger.info("==============================================================");

			Dataset<Row> teenagersDF2 = analysisLogic.process(args[0], requestQuery.toString());
			teenagersDF2.show(100);

			StringBuilder col1 = new StringBuilder();
			StringBuilder col2 = new StringBuilder();

			logger.info("==============================================================");
			logger.info("Column Count: "+teenagersDF2.columns().length);
			logger.info("==============================================================");

			String line;

			if(teenagersDF2.columns().length == 2) {
				teenagersDF2.collectAsList().stream()
						.forEach(data -> {
									col1.append(data.get(0).toString() + ",");
									col2.append(data.get(1).toString() + ",");
								}
						);
				line = col1.toString()+"\n"+ col2.toString();
			} else {
				teenagersDF2.collectAsList().stream()
						.forEach(data -> {
									col1.delete(0, col1.length());
									col1.append(data.get(0).toString() + "\t"
											+ data.get(1).toString() + "\t"
											+ data.get(2).toString() + "\t"
											+ data.get(3).toString() + "\t"
											+ data.get(4).toString() + "\t"
											+ data.get(5).toString() + "\t"
											+ data.get(6).toString() + "\t"
											+ data.get(7).toString()
									);

									col2.append(col1.toString()+"\n");
								}
						);
				line = col2.toString();
			}


			logger.info("==============================================================");
			logger.info("Result: "+line);
			logger.info("==============================================================");

//			Upload Result Output
			String encodeURL= "http://"+args[1]+":"+args[2]+"/upload?result=" + URLEncoder.encode( line, "UTF-8" );
			URL resUrl = new URL(encodeURL);
			HttpURLConnection resConn = (HttpURLConnection) resUrl.openConnection();
            resConn.setRequestMethod("POST");
			int responseCode = resConn.getResponseCode();

			logger.info("==============================================================");
			logger.info("Upload Success... : "+responseCode);
			logger.info("==============================================================");

		} catch (Exception ex) {
			logger.info("==============================================================");
			ex.printStackTrace();
			logger.info("==============================================================");
		}


	}


}